﻿using UnityEngine;
using UnityEngine.Rendering;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.IO;
using System;

public class CaptureScript : MonoBehaviour
{
    private RenderTexture renderTexture;
    private Texture2D screenShot;
    private Rect captureRect;
    public Shader effectsShader;

    public Camera renderCam;
    public int Width = 1368;
    public int Height = 912;

    private const string PATH = "D:/data/test";

    // Start is called before the first frame update
    void Start()
    {
        
    }

    // Update is called once per frame
    void Update()
    {
        
    }

    void LateUpdate()
    {
        //List<byte> img = GetFrameData(false);
        List<float> depth = GetFrameData();

        //string dn = PATH + "/depth/test.png";
        //string ds = PATH + "/scene/test.png";


        //File.WriteAllBytes(ds, img.ToArray());

        //float[] fl = depth.ToArray();

        //var byteArray = new byte[fl.Length * 4];
        //Buffer.BlockCopy(fl, 0, byteArray, 0, byteArray.Length);

        //File.WriteAllBytes(dn, byteArray);
    }

    private void SetUpRenderTextureForCapture()
    {
        renderTexture = RenderTexture.GetTemporary(Width, Height, 24, RenderTextureFormat.ARGB32);

        renderTexture.Create();
        renderCam.targetTexture = renderTexture;

        screenShot = new Texture2D(Width, Height, TextureFormat.RGB24, false);
        captureRect = new Rect(0, 0, Width, Height);
    }

    private void SetDepthEffect()
    {
        renderCam.renderingPath = RenderingPath.Forward;
        SetupCameraWithReplacementShader(2, Color.white);
    }

    private void ResetCameraEffects()
    {
        renderCam.renderingPath = RenderingPath.UsePlayerSettings;
        renderCam.clearFlags = CameraClearFlags.Skybox;
        renderCam.SetReplacementShader(null, null);
    }

    private void SetupCameraWithReplacementShader(int mode, Color clearColor)
    {
        var cb = new CommandBuffer();
        cb.SetGlobalFloat("_OutputMode", (int)mode);
        renderCam.AddCommandBuffer(CameraEvent.BeforeForwardOpaque, cb);
        renderCam.AddCommandBuffer(CameraEvent.BeforeFinalPass, cb);
        renderCam.SetReplacementShader(effectsShader, "");
        renderCam.backgroundColor = clearColor;
        renderCam.clearFlags = CameraClearFlags.SolidColor;
    }

    private List<byte> GetFrameData(bool isCompress)
    {
        SetDepthEffect();
        SetUpRenderTextureForCapture();

        RenderTexture.active = renderTexture;
        screenShot.ReadPixels(captureRect, 0, 0);
        screenShot.Apply();

        List<byte> frameData;

        if (isCompress)
        {
            byte[] bytes = screenShot.EncodeToPNG();
            frameData = new List<byte>(bytes);
        }
        else
        {
            Color32[] data = screenShot.GetPixels32();
            frameData = new List<byte>();
            foreach (Color32 c in data)
            {
                frameData.Add(c.r);
                frameData.Add(c.g);
                frameData.Add(c.b);
                frameData.Add(c.a);
            }
        }
        RenderTexture.active = null;
        return frameData;
    }

    private List<float> GetFrameData()
    {
        ResetCameraEffects();
        SetUpRenderTextureForCapture();
        RenderTexture.active = renderTexture;
        screenShot.ReadPixels(captureRect, 0, 0);
        screenShot.Apply();
        Color[] data = screenShot.GetPixels();
        //Debug.Log(data[3]);
        RenderTexture.active = null;
        List<float> frameData = new List<float>();
        foreach (Color c in data)
        {
            Debug.Log(c);
            frameData.Add(c.r);
            //frameData.Add(c.g);  //Need to Confirm with Microsoft regarding the other data, as Unreal is just sending the R data.
            //frameData.Add(c.b);
            //frameData.Add(c.a);
        }
        return frameData;
    }

}
