from pathlib import Path
import cv2
import imageio
import numpy as np
from multiprocessing import Pool
from enum import Enum

base_path = Path('D:/data/test')
depth_path2 = base_path / 'depth'
scene_path = base_path / 'scene'
label_path = base_path / 'truth'
depth_path1 = base_path / 'depth2'

LABEL_MAP = [([0, 0, 0], 0), ([0, 0, 255], 1), ([255, 235, 4], 2), ([255, 0, 0], 3)]


def build_path():
    choices = ['train', 'val', 'test']
    probs = [0.7, 0.2, 0.1]  # Probabilities of choosing 'train', 'val', or 'test'
    return base_path / np.random.choice(choices, p=probs)


def process_label(color_image):
    color_image = color_image[:, :, :3]
    label_image = np.zeros((color_image.shape[:2]), dtype=np.uint8)
    for m in LABEL_MAP:
        label_image[(color_image == m[0]).all(axis=2)] = m[1]
    return label_image


def process_depth(depth_img):
    depth_img = depth_img[:, :, :3]
    nimg = np.zeros((depth_img.shape[0], depth_img.shape[1], 1), dtype=np.uint16)
    for c in range(len(depth_img)):
        for r in range(len(depth_img[c])):
            v = depth_img[c][r][0] * 256 + depth_img[c][r][1]
            nimg[c][r] = v
    return nimg


def process_mix(stem):
    simg = cv2.imread(str(scene_path / (stem + '.png')))
    timg = cv2.imread(str(label_path / (stem + '.png')))
    return cv2.addWeighted(simg, 0.7, timg, 0.3, 0)


def check_paths(path):
    for p in ['train', 'val', 'test']:
        spath = path / p
        for p2 in ['scene', 'color', 'label', 'color_label', 'mix', 'depth', 'depth2', 'color_depth']:
            p3 = spath / p2
            p3.mkdir(parents=True, exist_ok=True)


def process_path(stem):
    path = build_path()

    # CV2 Mixed image
    mimg = process_mix(stem)

    # Imageio, depth and label image
    color_depth_image = imageio.imread(depth_path2 / (stem + '.png'))
    color_image = imageio.imread(label_path / (stem + '.png'))
    scene_image = imageio.imread(scene_path / (stem + '.png'))
    depth_image2 = process_depth(color_depth_image)
    depth_image1 = imageio.imread(depth_path1 / (stem + '.png'))
    label_image = process_label(color_image)

    imageio.imwrite(path / ('scene/' + stem + '.png'), scene_image)
    imageio.imwrite(path / ('depth2/' + stem + '.png'), depth_image2)
    imageio.imwrite(path / ('depth/' + stem + '.png'), depth_image1)
    imageio.imwrite(path / ('color_depth/' + stem + '.png'), color_depth_image)
    imageio.imwrite(path / ('label/' + stem + '.png'), label_image)
    imageio.imwrite(path / ('color_label/' + stem + '.png'), color_image)
    cv2.imwrite(str(path / ('mix/' + stem + '.png')), mimg)


def main():
    check_paths(base_path)
    stem_list = [p.stem for p in scene_path.iterdir()]

    pool = Pool(processes=32)
    pool.map(process_path, stem_list)



if __name__ == '__main__':
    print('test')
    main()
